class TreeNode:
    def __init__(self, value):
        self.value = value
        self.left = None
        self.right = None


class BinaryTree:
    def __init__(self):
        self.root = None
        self._size = 0

    def add(self, value):
        new_node = TreeNode(value)
        if not self.root:
            self.root = new_node
        else:
            queue = [self.root]
            while queue:
                current = queue.pop(0)
                if not current.left:
                    current.left = new_node
                    break
                elif not current.right:
                    current.right = new_node
                    break
                else:
                    queue.append(current.left)
                    queue.append(current.right)
        self._size += 1

    def remove(self, value):
        if not self.root:
            return False
        queue = [self.root]
        target = None
        last = None
        while queue:
            last = queue.pop(0)
            if last.value == value:
                target = last
            if last.left:
                queue.append(last.left)
            if last.right:
                queue.append(last.right)
        if target:
            target.value = last.value
            self._delete_deepest(last)
            self._size -= 1
            return True
        return False

    def _delete_deepest(self, node):
        queue = [self.root]
        while queue:
            current = queue.pop(0)
            if current.left:
                if current.left == node:
                    current.left = None
                    return
                queue.append(current.left)
            if current.right:
                if current.right == node:
                    current.right = None
                    return
                queue.append(current.right)

    def height(self):
        def _height(node):
            if not node:
                return 0
            return 1 + max(_height(node.left), _height(node.right))

        return _height(self.root)

    def size(self):
        return self._size

    def min(self):
        if not self.root:
            return None
        min_value = self.root.value
        queue = [self.root]
        while queue:
            current = queue.pop(0)
            min_value = min(min_value, current.value)
            if current.left:
                queue.append(current.left)
            if current.right:
                queue.append(current.right)
        return min_value

    def max(self):
        if not self.root:
            return None
        max_value = self.root.value
        queue = [self.root]
        while queue:
            current = queue.pop(0)
            max_value = max(max_value, current.value)
            if current.left:
                queue.append(current.left)
            if current.right:
                queue.append(current.right)
        return max_value

    def levelorder(self):
        result = []
        if not self.root:
            return result
        queue = [self.root]
        while queue:
            current = queue.pop(0)
            result.append(current.value)
            if current.left:
                queue.append(current.left)
            if current.right:
                queue.append(current.right)
        return result

    def inorder(self):
        def _inorder(node):
            if not node:
                return []
            return _inorder(node.left) + [node.value] + _inorder(node.right)

        return _inorder(self.root)

    def preorder(self):
        def _preorder(node):
            if not node:
                return []
            return [node.value] + _preorder(node.left) + _preorder(node.right)

        return _preorder(self.root)

    def postorder(self):
        def _postorder(node):
            if not node:
                return []
            return _postorder(node.left) + _postorder(node.right) + [node.value]

        return _postorder(self.root)

    def head(self):
        return self.root.value if self.root else None

    def leaf(self):
        leaves = []
        if not self.root:
            return leaves
        queue = [self.root]
        while queue:
            current = queue.pop(0)
            if not current.left and not current.right:
                leaves.append(current.value)
            if current.left:
                queue.append(current.left)
            if current.right:
                queue.append(current.right)
        return leaves
